<?php
namespace PME\Modules\Planejamento\Cadastro;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class CentralController extends \PME\Services\TeknisaCrud {

	protected $crudService;
	protected $parameterBag;
	protected $entityManager;
	protected $connection;
	protected $environment;
	protected $utilities;

    public function __construct($crudService, $parameterBag, $entityManager, $connection, $environment, $utilities) {
    	$this->entityManager = $entityManager;
    	$this->connection = $connection;
    	$this->environment = $environment;
    	$this->utilities = $utilities;
        parent::__construct($crudService, $parameterBag);
	}

	public function excluiCentral(DTO\Request $request, DTO\Response $response) {
		try {		
			$params = $request->getParameters()["checkedRows"];
            $this->connection->beginTransaction();
			
			foreach ($params as $param) {
				$validacao = $this->validaCentral($param);
				
				if ($validacao !== true) {
					$this->connection->rollBack();
					$response->addDataset(new DataSet('mensagem', array($validacao)));
					return;
				};
				
				$sql = "UPDATE FILIAL 
				           SET IDTPFILIALCENT = NULL, IDFILCENTPME = 'N',
				               CDFILCENTPME = NULL
                         WHERE CDFILIAL = :CDFILIAL ";
                
                $params = array(
                    'CDFILIAL' => $param['CDFILIAL']
                );
                 
                $this->entityManager->getConnection()->executeUpdate($sql, $params);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function salvaCentral(DTO\Request $request, DTO\Response $response) {
		try {			
			$params = $request->getParameters()["editedRows"];
            $this->connection->beginTransaction();
			
			foreach ($params as $param) {
				if ($param['__is_new'] === true) {
					$validacao = $this->validaUnidade($param);
					
					if ($validacao !== true) {
						$this->connection->rollBack();
						$response->addDataset(new DataSet('mensagem', array($validacao)));
						return;
					};
				}
			    
			    if ($param['IDTPFILIALCENT'] == 'M') {
			        $validaTipo = $this->validaCentralComMerenda($param);
			        if ($validaTipo !== true){
			            $this->connection->rollBack();
						$response->addDataset(new DataSet('mensagem', array( $validaTipo)));
						return;
			        }
			    }
			    
				// Atualiza unidade como central conforme o tipo passado.
				$sql = "UPDATE FILIAL 
				           SET IDTPFILIALCENT = :IDTPFILIALCENT, IDFILCENTPME = 'S' 
                         WHERE CDFILIAL = :CDFILIAL ";
                
                $paramsUp = array(
                    'CDFILIAL' => $param['CDFILIAL'],
                    'IDTPFILIALCENT' => $param['IDTPFILIALCENT']
                );
                
                $this->entityManager->getConnection()->executeUpdate($sql, $paramsUp);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
            $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function salvaAgregada(DTO\Request $request, DTO\Response $response) {
		try {			
			$params = $request->getParameters()["editedRows"];
            $this->connection->beginTransaction();
			$unidadesSemParamPlaneja = array();
			$unitysErrors = array();
			foreach ($params as $param) {
				$validacao = $this->validaAgregada($param, $unitysErrors);
				
				if ($validacao !== true) {
					$this->connection->rollBack();
					$response->addDataset(new DataSet('mensagem', array($validacao)));
					$response->addDataset(new DataSet('comp', array_unique($unitysErrors)));
					return;
				};
				
				$hasParamPlaneja = $this->utilities->checkParamPlaneja($param['CDFILIAL']);
				if ($hasParamPlaneja !== true) {
				    $unidadesSemParamPlaneja[] = $hasParamPlaneja;
				    continue;
				}
				
				$sql = "UPDATE FILIAL 
				           SET CDFILCENTPME = :CDFILCENTPME
                         WHERE CDFILIAL = :CDFILIAL ";
                
                $paramsUp = array(
                    'CDFILIAL' => $param['CDFILIAL'],
                    'CDFILCENTPME' => $param['CDFILCENTPME']
                );
                
                $this->entityManager->getConnection()->executeUpdate($sql, $paramsUp);
			}
			if (!empty($unidadesSemParamPlaneja)) {
			    $response->addDataset(new DataSet('mensagem', array('ERROR_AGREGADA_SEM_PARAMPLANEJA')));
			    $response->addDataset(new DataSet('comp', array_unique($unidadesSemParamPlaneja)));
			}
            $this->connection->commit();
        }catch (\Exception $e) {
            $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function validaUnidade($param){
		// Valida se unidade já é agregada de outra
		$sql = "SELECT A.CDFILIAL, A.CDFILCENTPME, C.NMFILIAL
				  FROM FILIAL A
                  INNER JOIN FILIAL C ON C.CDFILIAL = A.CDFILCENTPME 
                 WHERE A.CDFILIAL = :CDFILIAL ";
        
        $paramSql = array(
            'CDFILIAL' => $param['CDFILIAL']
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_E_AGREGADA:' . $resultado[0]['CDFILCENTPME'] . ' - ' . $resultado[0]['NMFILIAL'];
        }
        
        // Valida se unidade já é cadastrada como central
		$sql = "SELECT CDFILIAL, NMFILIAL
				  FROM FILIAL 
                 WHERE CDFILIAL = :CDFILIAL 
                   AND IDFILCENTPME = 'S'";
        
        $paramSql = array(
            'CDFILIAL' => $param['CDFILIAL']
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_JA_CADASTRADA:' . $resultado[0]['CDFILIAL'] . ' - ' . $resultado[0]['NMFILIAL'];
        }
        
        return true;
	}
	
	private function validaCentral($param){
		// Valida se central tem agregadas além dela
		$sql = "SELECT A.CDFILIAL, A.CDFILCENTPME, A.NMFILIAL
				  FROM FILIAL A
                 WHERE A.CDFILCENTPME = :CDFILIAL 
                   AND A.CDFILIAL <> A.CDFILCENTPME";
        
        $paramSql = array(
            'CDFILIAL' => $param['CDFILIAL']
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_TEM_AGREGADA';
        }
        
        return true;
	}
	
	// valida se a central do tipo merenda possui ela mesma como agregada
	private function validaCentralComMerenda($param) {
   		$sql = "SELECT CDFILIAL, NMFILIAL
			  FROM FILIAL 
             WHERE CDFILCENTPME = :CDFILIAL ";
            
        $paramSql = array(
            'CDFILIAL' => $param['CDFILIAL']
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        
        foreach ($resultado as $result) {
            if (strcasecmp($result['CDFILIAL'], $param['CDFILIAL']) == 0){
                return 'ERROR_CENTRAL_M';
            }
        }
        
        return true;
	}
	
	private function validaAgregada($param, &$unitysErrors){
	    if ($param['CDFILIAL'] <> $param['CDFILCENTPME']) { // Valida se unidade já é cadastrada como central e não é a atual
    // 		$sql = "SELECT CDFILIAL, NMFILIAL
    // 				  FROM FILIAL 
    //                  WHERE CDFILCENTPME = :CDFILIAL ";
            
    //         $paramSql = array(
    //             'CDFILIAL' => $param['CDFILIAL']
    //         );
            
            $sql2 = "SELECT
                        CDFILIAL, NMFILIAL
                     FROM 
                        FILIAL
                     WHERE 
                        IDFILCENTPME = :IDFILCENTPME";
                        
            $paramSql2 = array(
                'IDFILCENTPME' => 'S'
            );
            
            // $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
            
            $resultado2 = $this->entityManager->getConnection()->fetchAll($sql2, $paramSql2);
            
            foreach ($resultado2 as $result) {
                if (strcasecmp($result['CDFILIAL'], $param['CDFILIAL']) == 0){
                	$unitysErrors[] = $result['CDFILIAL'].' | '.$result['NMFILIAL'];
                    return 'ERROR_UNID_CENTRAL';
                }
            }
            
            // if (count($resultado) > 0) {
            // 	return 'ERROR_UNID_CENTRAL';
            // }
	    }
	    elseif ($param['IDTPFILIALCENT'] == 'M') {
	    	$unitysErrors[] = $param['CDFILIAL'].' | '.$param['NMFILIAL'];
	        return 'ERROR_AGREGADA_M';
	    }
        
        return true;
	}
	
	public function excluiAgregada(DTO\Request $request, DTO\Response $response) {
		try {		
			$params = $request->getParameters()["checkedRows"];
            $this->connection->beginTransaction();
			
			foreach ($params as $param) {
				$sql = "UPDATE FILIAL 
				           SET CDFILCENTPME = NULL 
                         WHERE CDFILIAL = :CDFILIAL ";
                
                $params = array(
                    'CDFILIAL' => $param['CDFILIAL']
                );
                
                $this->entityManager->getConnection()->executeUpdate($sql, $params);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}