import {
  Http, Loader, Metadata, I18n, Datasource, IEventParam, IDictionary,
} from '@zeedhi/core';

import {
  Form, Button, DialogService, GridEditable, Grid, Card, Modal, ModalService, Select,
} from '@zeedhi/common';
  
export class Ger96000CentralController {
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  private agregadaIsVisible = false;
  
  public selectedCentral = '';
  
  private modalRelatorioCentral!: Modal;
  
  public reportController = Loader.getInstance('ReportController');
  
  private isFormatting = false;

  public save({ component }: IDictionary) {
    const grid: GridEditable = Metadata.getInstance('CENTRAL');
    const editedRows = grid.getEditedRows(false, true);

    if (grid.isGridValid(true)) {
      if (editedRows.length > 0) {
        editedRows.forEach((row: IDictionary) => {
          if (!row.__is_new) {
            row.__is_new = false;
          }
        });
        
        if (grid) grid.datasource.loading = true;
        Http.post('/salvaCentral', { editedRows }).then((response: IDictionary) => {
          let param;
          if (response.data.length === 0) {
            grid.datasource.get();
            this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'success');
            this.gridEditableController.cancelEditing({ component });
          } else {
            const mensagem = response.data.dataset.mensagem[0];
            
            if (mensagem.indexOf('ERROR_E_AGREGADA') === 0) {
              param = mensagem.split(':');
              this.utilities.showAlert('alertError',
                I18n.translate('Unidade já está cadastrada como agregada para a central {{PARAM1}}',
                  { PARAM1: param[1] }), 'error');
            } else if (mensagem.indexOf('ERROR_JA_CADASTRADA') === 0) {
              param = mensagem.split(':');
              this.utilities.showAlert('alertError',
                I18n.translate('Unidade já está cadastrada como central.',
                  { PARAM1: param[1] }), 'error');
            } else if (mensagem.indexOf('ERROR_CENTRAL_M') === 0) {
              this.utilities.showAlert('alertError', 'Operação bloqueada! Central agregada a ela mesmo.', 'error');
              // this.gridEditableController.cancelEditing({ component });
            } else {
              this.utilities.showAlert('alertError', mensagem, 'error');
            }
          }
        }).catch((error) => {
          this.utilities.showAlert('alertError', error, 'error');
        }).finally(() => {
          if (grid) grid.datasource.loading = false;
        });
      } else {
        this.gridEditableController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }
  
  public cancelEditingTelaCentral({ component }: IDictionary) {
    this.gridEditableController.cancelEditing({ component });
  }
  
  private cardAgregadaVisible(column: any) {
    const card = Metadata.getInstance<Card>('agregadaCard');
    const gridCentral: GridEditable = Metadata.getInstance('CENTRAL');
    try {
      const columnEditable = column?.column?.editable || '';
      if (!gridCentral.editing && !this.gridEditableController.newRow
         && !this.gridEditableController.isInlineEditing && !columnEditable) {
        card.isVisible = true;
        this.agregadaIsVisible = true;
      } else {
        card.isVisible = false;
        this.agregadaIsVisible = false;
        // if (gridCentral.editing) this.gridEditableController.defaultState(gridCentral);
      }
    } catch (Error) {
      card.isVisible = false;
      this.agregadaIsVisible = false;
    }
  }
  
  public closeCardAgregada() {
    const card = Metadata.getInstance<Card>('agregadaCard');
    const gridAgregada: GridEditable = Metadata.getInstance('AGREGADA');
    this.gridEditableController.cancelEditing({ component: gridAgregada });
    
    card.isVisible = false;
    this.agregadaIsVisible = false;
  }
  
  get gridColsReturnValue(): number {
    return this.agregadaIsVisible ? 6 : 12;
  }

  public carregaCentrais() {
    const central: GridEditable = Metadata.getInstance('CENTRAL');
    central.datasource.get().then(() => {
      central.datasource.currentRow = central.datasource.data[0];
    });
  }
  
  public salvaAgregada({ component }: Button) {
    const form = Metadata.getInstance<Form>('formAgregada');
    const grid: GridEditable = Metadata.getInstance('AGREGADA');
    let editedRows = grid.getEditedRows();
    const selectUnidade = Metadata.getInstance<Select>('CDFILIAL_editable_');
    if (editedRows.length === 0) {
      editedRows = [{
        CDFILIAL: selectUnidade.value,
        __is_new: true,
      }];
    }
    const central: GridEditable = Metadata.getInstance('CENTRAL');
    const currentRow = central.datasource.currentRow;

    if (!currentRow.CDFILIAL) this.utilities.showAlert('alertError', 'Central não selecionada!', 'error');
    else if (form.validate()) {
      if (editedRows.length > 0) {
        const editedRowsToSend: any = [];
        Object.keys(editedRows[0]).forEach((key: any) => {
          if (Array.isArray(editedRows[0][key])) {
            editedRows[0][key].forEach((value: any, idx: number) => {
              if (editedRowsToSend[idx]) editedRowsToSend[idx][key] = value;
              else {
                editedRowsToSend[idx] = {};
                editedRowsToSend[idx][key] = value;
              }
            });
          }
        });
        editedRowsToSend.forEach((row: IDictionary) => {
          row.CDFILCENTPME = central.datasource.currentRow.CDFILIAL;
          row.IDTPFILIALCENT = central.datasource.currentRow.IDTPFILIALCENT;
        });
        
        if (grid) grid.datasource.loading = true;
        Http.post('/salvaAgregada', { editedRows: editedRowsToSend }).then((response: IDictionary) => {
          if (response.data.length === 0) {
            grid.datasource.get();
            
            this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'success');
            this.gridEditableController.cancelEditing({ component });
          } else if (response.data.dataset.mensagem[0] === 'ERROR_UNID_CENTRAL') {
            const unidades = response?.data?.dataset?.comp || [];
            this.utilities.showAlert('alertError',
              I18n.translate('Unidade já está cadastrada como central. {{PARAM1}}',
                { PARAM1: unidades?.join(', ') }), 'error');
          } else if (response.data.dataset.mensagem[0] === 'ERROR_AGREGADA_SEM_PARAMPLANEJA') {
            const unidades = response?.data?.dataset?.comp || [];
            this.utilities.showAlert('alertError',
              I18n.translate('pme::ERROR_AGREGADA_SEM_PARAMPLANEJA', { PARAM1: unidades?.join(', ') }), 'error');
          } else if (response.data.dataset.mensagem[0] === 'ERROR_AGREGADA_M') {
            const unidades = response?.data?.dataset?.comp || [];
            this.utilities.showAlert('alertError',
              I18n.translate('pme::ERROR_AGREGADA_M', { PARAM1: unidades?.join(', ') }), 'error');
          } else {
            this.utilities.showAlert('alertError',
              I18n.translate('Operação bloqueada! Unidade cadastrada como central do tipo merenda.'),
              'error');
          }
        }).catch((error) => {
          this.utilities.showAlert('alertError', error, 'error');
        }).finally(() => {
          if (grid) grid.datasource.loading = false;
        });
      } else {
        // this.gridEditableController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }
  
  public confirmDeleteCentral({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else {
      DialogService.show({
        name: 'excluiCentral',
        type: 'warning',
        title: 'Atenção',
        text: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.deleteCentral() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteCentral() {
    const gridCentral = Metadata.getInstance<GridEditable>('CENTRAL');
    const checkedRows = gridCentral.selectedRows;
    Http.post('/excluiCentral', { requestType: 'Row', checkedRows }).then((response: IDictionary) => {
      if (response.data.length === 0) {
        gridCentral.datasource.get();
        gridCentral.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'Excluído com sucesso!', 'success');
        DialogService.hide();
      } else {
        const mensagem = response.data.dataset.mensagem[0];
        this.utilities.showAlert('alertError', `pme::${mensagem}`, 'error');
        DialogService.hide();
      }
    }).catch((error) => {
      gridCentral.selectedRows = [];
      this.utilities.showAlert('alertError', error, 'error');
      DialogService.hide();
    }).finally(() => {
      this.closeCardAgregada();
    });
  }
  
  public confirmDeleteAgregada({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else {
      DialogService.show({
        name: 'excluiAgregada',
        type: 'warning',
        title: 'Atenção',
        text: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.deleteAgregada() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteAgregada() {
    const gridAgregada = Metadata.getInstance<GridEditable>('AGREGADA');
    const checkedRows = gridAgregada.selectedRows.map((row: IDictionary) => {
      Object.keys(row).forEach((key: any) => {
        if (Array.isArray(row[key])) row[key] = row[key][0];
      });
      return row;
    });
    Http.post('/excluiAgregada', { requestType: 'Row', checkedRows }).then(() => {
      gridAgregada.datasource.get();
      gridAgregada.selectedRows = [];
      this.utilities.showAlert('alertSuccess', 'Excluído com sucesso!', 'success');
      DialogService.hide();
    }).catch((error) => {
      gridAgregada.selectedRows = [];
      this.utilities.showAlert('alertError', error, 'error');
      DialogService.hide();
    });
  }

  public carregaAgregada({ component }: IEventParam<Datasource>) {
    const centralSelecionada = !!component.currentRow.CDFILIAL;

    if (centralSelecionada && !this.gridEditableController.isInlineEditing) {
      const agregada: Grid = Metadata.getInstance('AGREGADA');
      this.selectedCentral = `${component.currentRow.CDFILIAL} - ${component.currentRow.NMFILIAL}`;
    
      const conditions = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: component.currentRow.CDFILIAL,
        },
      ];
  
      agregada.datasource.filter = conditions;
      // this.cardAgregadaVisible({});
    } else {
      this.selectedCentral = `${component.currentRow.CDFILIAL} - ${component.currentRow.NMFILIAL}`;
      this.closeCardAgregada();
    }
  }
  
  public setFilterAgregada({ component }: IEventParam<Select>) {
    const gridCentral: GridEditable = Metadata.getInstance('CENTRAL');
    const central = gridCentral.datasource.currentRow.CDFILIAL;
    const tpCentral = gridCentral.datasource.currentRow.IDTPFILIALCENT;
    
    component.datasource.loading = true;
    
    let filter = [
      {
        name: 'P_CDCENTRAL',
        operator: '=',
        value: 'xxxx',
      },
    ];
    
    if (tpCentral === 'M') {
      filter = [
        {
          name: 'P_CDCENTRAL',
          operator: '=',
          value: central,
        },
      ];
    }
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  private async createModalRelatorioCentral() {
    this.modalRelatorioCentral = await ModalService.createFromJsonPost('',
      { jsonPath: 'planejamento/cadastro/modals/ger96000_relatorio_central' });
  }
  
  public openModalRelatorioCentral() {
    if (!this.modalRelatorioCentral) {
      this.createModalRelatorioCentral().then(() => {
        this.modalRelatorioCentral.show();
      });
    } else {
      this.modalRelatorioCentral.show();
    }
  }
  
  public cancelModalRelatorioCentral() {
    this.modalRelatorioCentral.hide();
  }
  
  private setParams(row: IDictionary) {
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORMREPORTCENTRAL');
    const row: IDictionary = form.value;
    
    this.setParams(row);
    
    this.reportController.callGenerateReport(component, row, 'report/centrais');
  }
  
  public changeCentralCurrentDetail({ component }: IEventParam<Datasource>) {
    if (!this.isFormatting) {
      component.loading = true;
      this.isFormatting = true;
      setTimeout(() => {
        this.formatData({ component });
        this.isFormatting = false;
        component.loading = false;
      }, 50);
    }
  }

  public async formatData({ component }: IEventParam<Datasource>) {
    if (!component.data || component.data.length === 0) return;
  
    const mappedData = await component.data.map((row: IDictionary) => {
      const data: any = {};
      Object.keys(row).forEach((key: any) => {
        if (row[key] && !Array.isArray(row[key])) data[key] = [row[key]];
        else data[key] = row[key];
      });
      return data;
    });
    await component.updateData(mappedData);
  }
}
