import {
  Loader, Metadata, IDictionary, Http, IEventParam, Datasource,
} from '@zeedhi/core';
  
import { GridEditable, Card } from '@zeedhi/common';
  
export class Pla15300CategoriaEmbalagensController {
  public gridEditController = Loader.getInstance('GridEditableController');
    
  public crudController = Loader.getInstance('CrudController');
    
  private categoriaEmbalagensCardVisible = false;
    
  public utilities = Loader.getInstance('Utilities');
    
  public saveCategoria() {
    const grid: GridEditable = Metadata.getInstance('gridCategoria');
    const editedRows = grid.getEditedRows(false, true);

    if (grid.isGridValid(true)) {
      if (editedRows.length > 0) {
        Http.post('/retornaCodigoCategoria', {}).then((response: IDictionary) => {
          let codigo = response.data.dataset.codigo[0].CODIGO;
          
          editedRows.forEach((row: IDictionary) => {
            if (row.__is_new && codigo !== null) {
              row.NRCATEMBALAG = codigo;
              codigo += 1;
            } else if (row.__is_new && codigo === null) row.NRCATEMBALAG = '1';
            row.CDCATEMBALAG = row.CDCATEMBALAG.padStart(5, '0');
          });
          
          this.crudController.saveRows(editedRows, 'catembalag', grid).then(() => {
            this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
          }).catch((error: string) => {
            if (error.includes('ORA-00001')) {
              this.utilities.showAlert('alert', 'Impossível adicionar. Registro já existe.', 'error');
            } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
          }).finally(() => {
            this.gridEditController.defaultState(grid);
          });
        }).catch((error) => {
          this.utilities.showAlert('alertError', error, 'error');
        });
      } else {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }

  private cardEmbalagensVisible(column?: any) {
    const card = Metadata.getInstance<Card>('categoriaEmbalagensCard');
    const gridCategoria = Metadata.getInstance<GridEditable>('gridCategoria');
  
    // Define quais colunas não devem abrir o card
    const excludedColumns = [
      'NMCATEMBALAG',
    ];
  
    try {
      const columnName = column.column.name;
  
      const shouldOpenCard = !excludedColumns.includes(columnName)
        && !this.gridEditController.newRow
        && !gridCategoria.editing;
  
      card.isVisible = shouldOpenCard;
      this.categoriaEmbalagensCardVisible = shouldOpenCard;
    } catch (error) {
      card.isVisible = false;
      this.categoriaEmbalagensCardVisible = false;
    }
  }

  public onChangeEmbalagem({ component }: IEventParam<Datasource>) {
    const colunaEmb = component.currentRow.NRCATEMBALAG;

    if (colunaEmb) {
      const gridEmbalagem = Metadata.getInstance<GridEditable>('categoriaEmbalagensGridDetail');
      
      const conditions = [
        {
          name: 'NRCATEMBALAG',
          operator: '=',
          value: component.currentRow.NRCATEMBALAG,
        },
      ];
      
      gridEmbalagem.datasource.filter = conditions;
      gridEmbalagem.datasource.get();
      // this.cardEmbalagemVisible();
    }
  }
  
  public closeCardCategoriaEmbalagens() {
    const card = Metadata.getInstance<Card>('categoriaEmbalagensCard');
    
    card.isVisible = false;
    this.categoriaEmbalagensCardVisible = false;
  }

  get gridColsReturnValue(): number {
    return this.categoriaEmbalagensCardVisible ? 6 : 12;
    this.reload();
  }

  public reload() {
    const grid = Metadata.getInstance<GridEditable>('gridCategoria');
    this.gridEditController.reload({ component: grid });
  }
}
